//+------------------------------------------------------------------+
//|                                                     Ichimoku.mq4 |
//|                   Copyright 2005-2014, MetaQuotes Software Corp. |
//|                                              http://www.mql4.com |
//+------------------------------------------------------------------+
#property copyright   "2005-2014, MetaQuotes Software Corp."
#property link        "http://www.mql4.com"
#property description "Ichimoku Kinko Hyo"
#property strict

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 Red          // Tenkan-sen
#property indicator_color2 Blue         // Kijun-sen
#property indicator_color3 Yellow       // Senkou Span A
#property indicator_color4 Thistle      // EnvelopeUpLevel1
#property indicator_color5 Thistle      // EnvelopeUpLevel2
#property indicator_color6 Thistle      // EnvelopeDownLevel1
#property indicator_color7 Thistle      // EnvelopeDownLevel2
//--- input parameters
input int     InpTenkan=9;   // Tenkan-sen
input int     InpKijun=26;   // Kijun-sen
input bool    DrawTenkan=False;
input bool    DrawKijun=False;
extern double EnvelopeLevel1=400;
extern double EnvelopeLevel2=600;
//--- buffers
double ExtTenkanBuffer[];
double ExtKijunBuffer[];
double ExtSpanA2_Buffer[];
double ExtEnvelopeUp1_Buffer[];
double ExtEnvelopeUp2_Buffer[];
double ExtEnvelopeDown1_Buffer[];
double ExtEnvelopeDown2_Buffer[];
//---
int    ExtBegin;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit(void)
  {
   IndicatorDigits(Digits);
//---
   SetIndexBuffer(0,ExtTenkanBuffer);
   SetIndexDrawBegin(0,InpTenkan-1);
   SetIndexLabel(0,"Tenkan Sen");
   if(DrawTenkan){SetIndexStyle(0,DRAW_LINE,STYLE_SOLID);}
   else{SetIndexStyle(0,DRAW_NONE);}

//---
   SetIndexBuffer(1,ExtKijunBuffer);
   SetIndexDrawBegin(1,InpKijun-1);
   SetIndexLabel(1,"Kijun Sen");
   if(DrawKijun){SetIndexStyle(1,DRAW_LINE,STYLE_SOLID);}
   else{SetIndexStyle(1,DRAW_NONE);}
//---
   ExtBegin=InpKijun;
   if(ExtBegin<InpTenkan)
      ExtBegin=InpTenkan;
//---
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,ExtSpanA2_Buffer);
   SetIndexDrawBegin(2,InpKijun+ExtBegin-1);
   SetIndexShift(2,InpKijun);
   SetIndexLabel(2,"Senkou Span A");
//---
   SetIndexStyle(3,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(3,ExtEnvelopeUp1_Buffer);
   SetIndexDrawBegin(3,InpKijun+ExtBegin-1);
   SetIndexShift(3,InpKijun);
   SetIndexLabel(3,"EnvelopeUp1");
   SetIndexStyle(4,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(4,ExtEnvelopeUp2_Buffer);
   SetIndexDrawBegin(4,InpKijun+ExtBegin-1);
   SetIndexShift(4,InpKijun);
   SetIndexLabel(4,"EnvelopeUp2");
   SetIndexStyle(5,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(5,ExtEnvelopeDown1_Buffer);
   SetIndexDrawBegin(5,InpKijun+ExtBegin-1);
   SetIndexShift(5,InpKijun);
   SetIndexLabel(5,"EnvelopeDown1");
   SetIndexStyle(6,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(6,ExtEnvelopeDown2_Buffer);
   SetIndexDrawBegin(6,InpKijun+ExtBegin-1);
   SetIndexShift(6,InpKijun);
   SetIndexLabel(6,"EnvelopeDown2");
//--- initialization done
  }
//+------------------------------------------------------------------+
//| Ichimoku Kinko Hyo                                               |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   int    i,k,pos;
   double high_value,low_value;
//---
   if(rates_total<=InpTenkan || rates_total<=InpKijun)
      return(0);
//--- counting from 0 to rates_total
   ArraySetAsSeries(ExtTenkanBuffer,false);
   ArraySetAsSeries(ExtKijunBuffer,false);
   ArraySetAsSeries(ExtSpanA2_Buffer,false);
   ArraySetAsSeries(ExtEnvelopeUp1_Buffer,false);
   ArraySetAsSeries(ExtEnvelopeUp2_Buffer,false);
   ArraySetAsSeries(ExtEnvelopeDown1_Buffer,false);
   ArraySetAsSeries(ExtEnvelopeDown2_Buffer,false);
   ArraySetAsSeries(open,false);
   ArraySetAsSeries(high,false);
   ArraySetAsSeries(low,false);
   ArraySetAsSeries(close,false);
//--- initial zero
   if(prev_calculated<1)
     {
      for(i=0; i<InpTenkan; i++)
         ExtTenkanBuffer[i]=0.0;
      for(i=0; i<InpKijun; i++)
         ExtKijunBuffer[i]=0.0;
      for(i=0; i<ExtBegin; i++)
        {
         ExtSpanA2_Buffer[i]=0.0;
        }
     }
//--- Tenkan Sen
   pos=InpTenkan-1;
   if(prev_calculated>InpTenkan)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
      high_value=high[i];
      low_value=low[i];
      k=i+1-InpTenkan;
      while(k<=i)
        {
         if(high_value<high[k])
            high_value=high[k];
         if(low_value>low[k])
            low_value=low[k];
         k++;
        }
      ExtTenkanBuffer[i]=(high_value+low_value)/2;
     }
//--- Kijun Sen
   pos=InpKijun-1;
   if(prev_calculated>InpKijun)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
      high_value=high[i];
      low_value=low[i];
      k=i+1-InpKijun;
      while(k<=i)
        {
         if(high_value<high[k])
            high_value=high[k];
         if(low_value>low[k])
            low_value=low[k];
         k++;
        }
      ExtKijunBuffer[i]=(high_value+low_value)/2;
     }
//--- Senkou Span A
   pos=ExtBegin-1;
   if(prev_calculated>ExtBegin)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
      ExtSpanA2_Buffer[i]=(ExtKijunBuffer[i]+ExtTenkanBuffer[i])/2;
     }
//--- Pivot1
   pos=ExtBegin-1;
   if(prev_calculated>ExtBegin)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
     ExtEnvelopeUp1_Buffer[i]=((ExtKijunBuffer[i]+ExtTenkanBuffer[i])/2)+EnvelopeLevel1*Point;
     }
//--- Pivot2
   pos=ExtBegin-1;
   if(prev_calculated>ExtBegin)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
     ExtEnvelopeUp2_Buffer[i]=((ExtKijunBuffer[i]+ExtTenkanBuffer[i])/2)+EnvelopeLevel2*Point;
     }
//--- Pivot3
   pos=ExtBegin-1;
   if(prev_calculated>ExtBegin)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
     ExtEnvelopeDown1_Buffer[i]=((ExtKijunBuffer[i]+ExtTenkanBuffer[i])/2)-EnvelopeLevel1*Point;
     }
//--- Pivot4
   pos=ExtBegin-1;
   if(prev_calculated>ExtBegin)
      pos=prev_calculated-1;
   for(i=pos; i<rates_total; i++)
     {
     ExtEnvelopeDown2_Buffer[i]=((ExtKijunBuffer[i]+ExtTenkanBuffer[i])/2)-EnvelopeLevel2*Point;
     }
//---
   return(rates_total);
  }
//+------------------------------------------------------------------+
